@@ -10,20 +10,31 @@
 
 #include <chopper/configuration.hpp>
 #include <chopper/layout/ibf_query_cost.hpp>
+#include <chopper/layout/layout.hpp>
 #include <chopper/next_multiple_of_64.hpp>
 #include <chopper/sketch/hyperloglog.hpp>
 #include <chopper/sketch/toolbox.hpp>
+#include <chopper/workarounds.hpp>
 
-/*!\brief Workaround bogus memcpy errors in GCC 12.1 and 12.2. (Wrestrict and Wstringop-overflow)
- * \see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=105545
- */
-#ifndef CHOPPER_WORKAROUND_GCC_BOGUS_MEMCPY
-#    if defined(__GNUC__) && !defined(__llvm__) && !defined(__INTEL_COMPILER) && (__GNUC__ == 12 && __GNUC_MINOR__ < 3)
-#        define CHOPPER_WORKAROUND_GCC_BOGUS_MEMCPY 1
-#    else
-#        define CHOPPER_WORKAROUND_GCC_BOGUS_MEMCPY 0
-#    endif
-#endif
+namespace std
+{
+
+template <>
+struct hash<std::vector<size_t>>
+{
+    size_t operator()(std::vector<size_t> const & vec) const
+    {
+        return std::accumulate(vec.begin(),
+                               vec.end(),
+                               vec.size(),
+                               [](std::size_t hash, size_t value)
+                               {
+                                   return hash ^= value + 0x9e3779b9ULL + (hash << 6) + (hash >> 2);
+                               });
+    }
+};
+
+} // namespace std
 
 namespace chopper::layout
 {
@@ -41,13 +52,17 @@ public:
     /*!\brief Construct an empty HIBF with an empty top level IBF
      * \param[in] config_ User configuration for the HIBF.
      * \param[in] fp_correction_ The false positive correction factors to use for the statistics.
+     * \param[in] sketches_ The sketches of the input.
      * \param[in] kmer_counts The original user bin weights (kmer counts).
      */
     hibf_statistics(configuration const & config_,
                     std::vector<double> const & fp_correction_,
+                    std::vector<sketch::hyperloglog> const & sketches_,
                     std::vector<size_t> const & kmer_counts) :
         config{config_},
         fp_correction{&fp_correction_},
+        sketches{sketches_},
+        counts{kmer_counts},
         total_kmer_count{std::accumulate(kmer_counts.begin(), kmer_counts.end(), size_t{})}
     {}
 
@@ -61,9 +76,6 @@ public:
 
         //!\brief The query cost to arrive at this IBF (updated before backtracking respective DP).
         double current_query_cost{0.0};
-
-        //!\brief A pointer to the filenames of the user input sequences.
-        std::vector<std::string> filenames;
     };
 
     //!\brief The kind of bin that is stored.
@@ -77,12 +89,14 @@ public:
     class bin
     {
     public:
-        bin_kind const kind;            //!< Either a split or merged bin.
-        size_t const cardinality;       //!< The size/weight of the bin (either a kmer count or hll sketch estimation).
-        size_t const num_contained_ubs; //!< [MERGED] How many UBs are merged within this TB.
-        size_t const num_spanning_tbs;  //!< [SPLIT] How many TBs are used for this sindle UB.
-
-        level child_level; //!< [MERGED] The lower level ibf statistics.
+        bin_kind kind;            //!< Either a split or merged bin.
+        size_t cardinality;       //!< The size/weight of the bin (either a kmer count or hll sketch estimation).
+        size_t num_contained_ubs; //!< [MERGED] How many UBs are merged within this TB.
+        size_t num_spanning_tbs;  //!< [SPLIT] How many TBs are used for this sindle UB.
+        std::vector<size_t> user_bin_indices; //!< The user bin indices of this bin.
+        size_t tb_index;                      // The (first) technical bin idx this bin is stored in.
+        level child_level;                    //!< [MERGED] The lower level ibf statistics.
+        size_t child_level_idx;               //!< [MERGED] The lower level ibf statistics.
 
         bin() = default;                        //!< Defaulted.
         bin(bin const & b) = default;           //!< Defaulted.
@@ -91,11 +105,11 @@ public:
         bin & operator=(bin &&) = default;      //!< Defaulted.
         ~bin() = default;                       //!< Defaulted.
 
-        bin(bin_kind const kind_, size_t const card, size_t const contained_ubs, size_t const spanning_tbs) :
+        bin(bin_kind const kind_, size_t const spanning_tbs, std::vector<size_t> const & user_bin_indices_) :
             kind{kind_},
-            cardinality{card},
-            num_contained_ubs{contained_ubs},
-            num_spanning_tbs{spanning_tbs}
+            num_contained_ubs{user_bin_indices_.size()},
+            num_spanning_tbs{spanning_tbs},
+            user_bin_indices{user_bin_indices_}
         {
             assert((kind == bin_kind::split && num_contained_ubs == 1u)
                    || (kind == bin_kind::merged && num_spanning_tbs == 1u));
@@ -105,6 +119,10 @@ public:
     //!\brief Gather all statistics to have all members ready.
     void finalize()
     {
+        collect_bins();
+
+        compute_cardinalities(top_level_ibf);
+
         compute_total_query_cost(top_level_ibf);
 
         gather_statistics(top_level_ibf, 0);
@@ -396,6 +414,9 @@ public:
     //!\brief The estimated query cost relative to the total k-mer count in the data set.
     double expected_HIBF_query_cost{0.0};
 
+    //!\brief A reference to the input counts.
+    chopper::layout::layout hibf_layout;
+
 private:
     //!\brief Copy of the user configuration for this HIBF.
     configuration const config{};
@@ -403,6 +424,12 @@ private:
     //!\brief The false positive correction factors to use for the statistics.
     std::vector<double> const * const fp_correction{nullptr};
 
+    //!\brief A reference to the input sketches.
+    std::vector<sketch::hyperloglog> const & sketches;
+
+    //!\brief A reference to the input counts.
+    std::vector<size_t> const & counts;
+
     //!\brief The original kmer count of all user bins.
     size_t const total_kmer_count{};
 
@@ -459,6 +486,98 @@ private:
         return byte_size_to_formatted_str(size_in_bytes);
     }
 
+    void collect_bins()
+    {
+        std::vector<hibf_statistics::level> ibfs(hibf_layout.max_bins.size() + 1); // 0 = top_level
+        robin_hood::unordered_map<std::vector<size_t>, size_t> id_to_pos{};
+
+        // fill id_to_pos map
+        id_to_pos[std::vector<size_t>{}] = 0;
+        for (size_t i = 0; i < hibf_layout.max_bins.size(); ++i)
+            id_to_pos[hibf_layout.max_bins[i].previous_TB_indices] = i + 1;
+
+        for (auto const & user_bin_info : hibf_layout.user_bins)
+        {
+            std::vector<size_t> prev{};
+
+            // add user bin index to previous merged bins
+            for (size_t i = 0; i < user_bin_info.previous_TB_indices.size(); ++i)
+            {
+                auto & ibf = ibfs[id_to_pos.at(prev)];
+                auto const target_tb_index = user_bin_info.previous_TB_indices[i];
+
+                bool found_merged_bin{false};
+                for (auto & previous_bins_to_check : ibf.bins)
+                {
+                    if (previous_bins_to_check.tb_index == target_tb_index)
+                    {
+                        found_merged_bin = true;
+                        previous_bins_to_check.user_bin_indices.push_back(user_bin_info.idx);
+                        ++previous_bins_to_check.num_contained_ubs;
+                    }
+                }
+
+                if (!found_merged_bin)
+                {
+                    ibf.bins.emplace_back(hibf_statistics::bin_kind::merged, 1, std::vector<size_t>{user_bin_info.idx});
+                    ibf.bins.back().tb_index = target_tb_index;
+                    auto next = prev;
+                    next.push_back(target_tb_index);
+                    ibf.bins.back().child_level_idx = id_to_pos.at(next);
+                }
+                prev.push_back(target_tb_index);
+            }
+
+            // emplace a split bin at last since every user bin is on its lowest level single or split
+            auto & ibf = ibfs[id_to_pos.at(prev)];
+            ibf.bins.emplace_back(hibf_statistics::bin_kind::split,
+                                  user_bin_info.number_of_technical_bins,
+                                  std::vector<size_t>{user_bin_info.idx});
+            ibf.bins.back().tb_index = user_bin_info.storage_TB_id;
+        }
+
+        for (auto & ibf : ibfs)
+            for (auto & bin : ibf.bins)
+                if (bin.kind == hibf_statistics::bin_kind::merged)
+                    bin.child_level = ibfs[bin.child_level_idx];
+
+        top_level_ibf = std::move(ibfs[0]);
+    }
+
+    void compute_cardinalities(level & curr_level)
+    {
+        for (bin & current_bin : curr_level.bins)
+        {
+            if (current_bin.kind == bin_kind::merged)
+            {
+                if (config.disable_estimate_union)
+                {
+                    size_t sum{};
+                    for (size_t i = 0; i < current_bin.user_bin_indices.size(); ++i)
+                        sum += counts[current_bin.user_bin_indices[i]]; // TODO should be kmer_counts
+                    current_bin.cardinality = sum;
+                }
+                else
+                {
+                    assert(!current_bin.user_bin_indices.empty());
+                    sketch::hyperloglog hll = sketches[current_bin.user_bin_indices[0]];
+
+                    for (size_t i = 1; i < current_bin.user_bin_indices.size(); ++i)
+                        hll.merge(sketches[current_bin.user_bin_indices[i]]);
+
+                    current_bin.cardinality = hll.estimate();
+                }
+
+                compute_cardinalities(current_bin.child_level);
+            }
+            else if (current_bin.kind == bin_kind::split) // bin_kind::split
+            {
+                assert(current_bin.user_bin_indices.size() == 1);
+                current_bin.cardinality = counts[current_bin.user_bin_indices[0]];
+            }
+        }
+    }
+
     //!\brief Computes the estimated query cost
     void compute_total_query_cost(level & curr_level)
     {
@@ -479,14 +598,11 @@ private:
                 if (!config.disable_estimate_union)
                 {
                     // compute merged_bin_sketch
-                    assert(!current_bin.child_level.filenames.empty());
-                    std::vector<sketch::hyperloglog> sketches;
-                    sketch::toolbox::read_hll_files_into(config.sketch_directory,
-                                                         current_bin.child_level.filenames,
-                                                         sketches);
-                    sketch::hyperloglog hll = sketches[0];
-                    for (size_t i = 1; i < sketches.size(); ++i)
-                        hll.merge(sketches[i]);
+                    assert(!current_bin.user_bin_indices.empty());
+                    sketch::hyperloglog hll = sketches[current_bin.user_bin_indices[0]];
+
+                    for (size_t i = 1; i < current_bin.user_bin_indices.size(); ++i)
+                        hll.merge(sketches[current_bin.user_bin_indices[i]]);
 
                     merged_bin_sketches.push_back(std::move(hll));
                 }
