@@ -63,15 +63,17 @@ public:
      * \param[in] argc_ The number of command line arguments.
      * \param[in] argv_ The command line arguments to parse.
      */
-    format_parse(int const argc_, std::vector<std::string> argv_) : argc{argc_ - 1}, argv{std::move(argv_)}
-    {}
+    format_parse(int const argc_, std::vector<std::string> argv_) : argv{std::move(argv_)}
+    {
+        (void)argc_;
+    }
     //!\}
 
     /*!\brief Adds an sharg::detail::get_option call to be evaluated later on.
      * \copydetails sharg::parser::add_option
      */
-    template <typename option_type, typename config_type>
-    void add_option(option_type & value, config_type const & config)
+    template <typename option_type, typename validator_t>
+    void add_option(option_type & value, config<validator_t> const & config)
     {
         option_calls.push_back(
             [this, &value, config]()
@@ -83,8 +85,8 @@ public:
     /*!\brief Adds a get_flag call to be evaluated later on.
      * \copydetails sharg::parser::add_flag
      */
-    template <typename config_type>
-    void add_flag(bool & value, config_type const & config)
+    template <typename validator_t>
+    void add_flag(bool & value, config<validator_t> const & config)
     {
         flag_calls.push_back(
             [this, &value, config]()
@@ -96,8 +98,8 @@ public:
     /*!\brief Adds a get_positional_option call to be evaluated later on.
      * \copydetails sharg::parser::add_positional_option
      */
-    template <typename option_type, typename config_type>
-    void add_positional_option(option_type & value, config_type const & config)
+    template <typename option_type, typename validator_t>
+    void add_positional_option(option_type & value, config<validator_t> const & config)
     {
         positional_option_calls.push_back(
             [this, &value, config]()
@@ -678,8 +680,8 @@ private:
      * - throws on (mis)use of both identifiers for non-container type values,
      * - re-throws the validation exception with appended option information.
      */
-    template <typename option_type, typename config_type>
-    void get_option(option_type & value, config_type const & config)
+    template <typename option_type, typename validator_t>
+    void get_option(option_type & value, config<validator_t> const & config)
     {
         bool short_id_is_set{get_option_by_id(value, config.short_id)};
         bool long_id_is_set{get_option_by_id(value, config.long_id)};
@@ -811,8 +813,6 @@ private:
     std::vector<std::function<void()>> positional_option_calls;
     //!\brief Keeps track of the number of specified positional options.
     unsigned positional_option_count{0};
-    //!\brief Number of command line arguments.
-    int argc;
     //!\brief Vector of command line arguments.
     std::vector<std::string> argv;
     //!\brief Artificial end of argv if \-- was seen.
