@@ -20,6 +20,7 @@
 #include <sharg/detail/format_html.hpp>
 #include <sharg/detail/format_man.hpp>
 #include <sharg/detail/format_parse.hpp>
+#include <sharg/detail/format_tdl.hpp>
 #include <sharg/detail/version_check.hpp>
 
 namespace sharg
@@ -191,12 +192,6 @@ public:
         version_check_dev_decision{version_updates},
         subcommands{std::move(subcommands)}
     {
-        if (!std::regex_match(app_name, app_name_regex))
-        {
-            throw design_error{("The application name must only contain alpha-numeric characters or '_' and '-' "
-                                "(regex: \"^[a-zA-Z0-9_-]+$\").")};
-        }
-
         for (auto & sub : this->subcommands)
         {
             if (!std::regex_match(sub, app_name_regex))
@@ -406,6 +401,14 @@ public:
         if (parse_was_called)
             throw design_error("The function parse() must only be called once!");
 
+        // Before creating the detail::version_checker, we have to make sure that
+        // malicious code cannot be injected through the app name.
+        if (!std::regex_match(info.app_name, app_name_regex))
+        {
+            throw design_error{("The application name must only contain alpha-numeric characters or '_' and '-' "
+                                "(regex: \"^[a-zA-Z0-9_-]+$\").")};
+        }
+
         detail::version_checker app_version{info.app_name, info.version, info.url};
 
         if (std::holds_alternative<detail::format_parse>(format) && !subcommands.empty() && sub_parser == nullptr)
@@ -430,14 +433,24 @@ public:
             version_check_future = app_version_prom.get_future();
             app_version(std::move(app_version_prom));
         }
-
         std::visit(
-            [this](auto & f)
+            [this]<typename T>(T & f)
             {
-                f.parse(info);
+                if constexpr (std::same_as<T, detail::format_tdl>)
+                {
+                    f.parse(info, executable_name);
+                }
+                else
+                {
+                    f.parse(info);
+                }
             },
             format);
         parse_was_called = true;
+
+        // Exit after parsing any special format.
+        if (!std::holds_alternative<detail::format_parse>(format))
+            std::exit(EXIT_SUCCESS);
     }
 
     /*!\brief Returns a reference to the sub-parser instance if
@@ -704,6 +717,7 @@ private:
                  detail::format_version,
                  detail::format_html,
                  detail::format_man,
+                 detail::format_tdl,
                  detail::format_copyright/*,
                  detail::format_ctd*/> format{detail::format_help{{}, {}, false}}; // Will be overwritten in any case.
 
@@ -742,11 +756,11 @@ private:
      * - <b>\--version</b> sets the format to sharg::detail::format_version.
      * - <b>\--export-help html</b> sets the format to sharg::detail::format_html.
      * - <b>\--export-help man</b> sets the format to sharg::detail::format_man.
-     * - <b>\--export-help ctd</b> sets the format to sharg::detail::format_ctd.
+     * - <b>\--export-help cwl</b> sets the format to sharg::detail::format_tdl{FileFormat::CWL}.
+     * - <b>\--export-help ctd</b> sets the format to sharg::detail::format_tdl{FileFormat::CTD}.
      * - else the format is that to sharg::detail::format_parse
      *
-     * If `--export-help` is specified with a value other than html/man or ctd
-     * an sharg::parser_error is thrown.
+     * If `--export-help` is specified with a value other than html, man, cwl or ctd, an sharg::parser_error is thrown.
      */
     void init(int argc, char const * const * const argv)
     {
@@ -814,20 +828,20 @@ private:
                 {
                     if (argv_len <= i + 1)
                         throw too_few_arguments{"Option --export-help must be followed by a value."};
-                    export_format = {argv[i + 1]};
+                    export_format = std::string{argv[i + 1]};
                 }
 
                 if (export_format == "html")
                     format = detail::format_html{subcommands, version_check_dev_decision};
                 else if (export_format == "man")
                     format = detail::format_man{subcommands, version_check_dev_decision};
-                // TODO (smehringer) use when CTD support is available
-                // else if (export_format == "ctd")
-                //     format = detail::format_ctd{};
+                else if (export_format == "ctd")
+                    format = detail::format_tdl{detail::format_tdl::FileFormat::CTD};
+                else if (export_format == "cwl")
+                    format = detail::format_tdl{detail::format_tdl::FileFormat::CWL};
                 else
                     throw validation_error{"Validation failed for option --export-help: "
-                                           "Value must be one of [html, man]"};
-
+                                           "Value must be one of [html, man, ctd, cwl]."};
                 special_format_was_set = true;
             }
             else if (arg == "--copyright")
@@ -924,7 +938,8 @@ private:
     }
 
     //!brief Verify the configuration given to a sharg::parser::add_option call.
-    void verify_option_config(config<auto> const & config)
+    template <typename validator_t>
+    void verify_option_config(config<validator_t> const & config)
     {
         if (sub_parser != nullptr)
             throw design_error{"You may only specify flags for the top-level parser."};
@@ -936,7 +951,8 @@ private:
     }
 
     //!brief Verify the configuration given to a sharg::parser::add_flag call.
-    void verify_flag_config(config<auto> const & config)
+    template <typename validator_t>
+    void verify_flag_config(config<validator_t> const & config)
     {
         verify_identifiers(config.short_id, config.long_id);
 
@@ -945,7 +961,8 @@ private:
     }
 
     //!brief Verify the configuration given to a sharg::parser::add_positional_option call.
-    void verify_positional_option_config(config<auto> const & config) const
+    template <typename validator_t>
+    void verify_positional_option_config(config<validator_t> const & config) const
     {
         if (config.short_id != '\0' || config.long_id != "")
             throw design_error{"Positional options are identified by their position on the command line. "
