@@ -32,7 +32,8 @@ std::string const basic_options_str = "  Common options\n"
                                       "    --copyright\n"
                                       "          Prints the copyright/license information.\n"
                                       "    --export-help (std::string)\n"
-                                      "          Export the help page information. Value must be one of [html, man].\n"
+                                      "          Export the help page information. Value must be one of [html, man,\n"
+                                      "          ctd, cwl].\n"
                                       "    --version-check (bool)\n"
                                       "          Whether to check for the newest app version. Default: true.\n";
 
@@ -44,7 +45,7 @@ std::string const basic_version_str = "VERSION\n"
 
 std::string license_text()
 {
-    std::ifstream license_file{std::string{{SHARG_TEST_LICENSE_DIR}} + "/LICENSE.md"};
+    std::ifstream license_file{std::string{SHARG_TEST_LICENSE_DIR} + "/LICENSE.md"};
     EXPECT_TRUE(license_file) << "Could not open file '" SHARG_TEST_LICENSE_DIR "/LICENSE.md'";
     std::stringstream buffer;
     buffer << license_file.rdbuf();
