@@ -23,7 +23,8 @@ std::string const basic_options_str = "  Common options\n"
                                       "    --copyright\n"
                                       "          Prints the copyright/license information.\n"
                                       "    --export-help (std::string)\n"
-                                      "          Export the help page information. Value must be one of [html, man].\n";
+                                      "          Export the help page information. Value must be one of [html, man,\n"
+                                      "          ctd, cwl].\n";
 
 std::string const basic_version_str = "VERSION\n"
                                       "    Last update:\n"
@@ -212,6 +213,11 @@ TEST(validator_test, output_file)
             EXPECT_NO_THROW(my_validator5(not_existing_path));
         }
 
+        { // filepath is a directory. Checking writeability would delete content of directory.
+            sharg::output_file_validator my_validator{sharg::output_file_open_options::open_or_create};
+            EXPECT_THROW(my_validator(std::filesystem::temp_directory_path()), sharg::validation_error);
+        }
+
         { // file does exist & overwriting is prohibited
             sharg::output_file_validator my_validator{sharg::output_file_open_options::create_new, formats};
             EXPECT_THROW(my_validator(existing_path), sharg::validation_error);
@@ -1036,6 +1042,39 @@ TEST(validator_test, value_list_validator_error)
     EXPECT_THROW(parser5.parse(), sharg::validation_error);
 }
 
+// https://github.com/seqan/sharg-parser/issues/178
+TEST(validator_test, value_list_validator_issue178)
+{
+    std::filesystem::path option_value;
+    std::vector<std::filesystem::path> option_vector;
+    constexpr std::array valid_values{"ha", "ba", "ma"};
+
+    // option
+    char const * argv[] = {"./parser_test", "-s", "ba"};
+    sharg::parser parser{"test_parser", 3, argv, sharg::update_notifications::off};
+    sharg::detail::test_accessor::set_terminal_width(parser, 80);
+    parser.add_option(option_value,
+                      sharg::config{.short_id = 's', .validator = sharg::value_list_validator{valid_values}});
+
+    testing::internal::CaptureStderr();
+    EXPECT_NO_THROW(parser.parse());
+    EXPECT_TRUE((testing::internal::GetCapturedStderr()).empty());
+    EXPECT_EQ(option_value, "ba");
+
+    // option - vector
+    char const * argv2[] = {"./parser_test", "-s", "ha", "-s", "ba"};
+    sharg::parser parser2{"test_parser", 5, argv2, sharg::update_notifications::off};
+    sharg::detail::test_accessor::set_terminal_width(parser2, 80);
+    parser2.add_option(option_vector,
+                       sharg::config{.short_id = 's', .validator = sharg::value_list_validator{valid_values}});
+
+    testing::internal::CaptureStderr();
+    EXPECT_NO_THROW(parser2.parse());
+    EXPECT_TRUE((testing::internal::GetCapturedStderr()).empty());
+    EXPECT_EQ(option_vector[0], "ha");
+    EXPECT_EQ(option_vector[1], "ba");
+}
+
 TEST(validator_test, regex_validator_success)
 {
     std::string option_value;
